import BaseCurd from './BaseCurd'
import MapBuilder from './MapBuilder'
import bindPrototype from './bindPrototype'

export default (target, hasPagination = true) => ({
  name: 'BaseCurdList',
  extends: BaseCurd(target),
  data: () => ({
    optionGroup: Object.freeze({}),
    query: target.$$getQuery(),
    pagination: {
      pageNo: 0,
      pageSize: 20
    },
    data: Object.freeze([]),
    total: 0,
    selectedData: null,
    selectedList: Object.freeze([])
  }),
  methods: {
    afterLoadList () {
      this.selectedList = Object.freeze([])
      this.selectedData = null
      setTimeout(() => {
        this.$refs.table && this.$refs.table.doLayout()
      }, 0)
    },
    loadList () {
      let query = Object.assign({}, this.query)
      if (hasPagination) {
        Object.assign(query, this.pagination)
        this.$$request(target.$$api.getPagination, this.query, this.pagination)
          .then(data => {
            this.total = data.count
            this.data = Object.freeze(bindPrototype(data.list, target))
            this.afterLoadList()
          }).catch(console.error).finally(() => {})
      } else {
        this.$$request(target.$$api.getList, this.query).then(list => {
          this.total = list.length
          this.data = Object.freeze(bindPrototype(Object.freeze(list), target))
          this.afterLoadList()
        }).catch(console.error).finally(() => {})
      }
    },
    currentPageNoChange () {
      this.loadList()
    },
    onSearchBtnClick () {
      this.pagination.pageNo = 1
      this.loadList()
    },
    onSelectionChange (selection) {
      this.selectedList = Object.freeze(selection)
    },
    onRowClick (data) {
      this.selectedData = data
    },
    getOptions (optionTargetConfigs, optionGroup) {
      return Promise.all(
        optionTargetConfigs.map(
          optionTargetConfig => {
            const optionTarget = optionTargetConfig.target
            return this.$$request(optionTarget.$$api.getList, typeof optionTargetConfig.getQuery === 'function' ? optionTargetConfig.getQuery(optionGroup) : undefined).then(options => {
              optionGroup[optionTarget.name] = {
                list: options,
                map: MapBuilder.getObjectMap(options, optionTarget.$$idProp),
                target: optionTarget
              }

              const { setQuery } = optionTargetConfig
              typeof setQuery === 'function' && setQuery(options)
            })
          })
      )
    },
    async getOptionGroup () {
      const optionGroup = {}

      const { $$optionTargetConfigGroup } = target
      if (Array.isArray($$optionTargetConfigGroup)) {
        const $$optionTargetConfigGroupLen = $$optionTargetConfigGroup.length
        for (let i = 0; i < $$optionTargetConfigGroupLen; i++) {
          const optionTargetConfigs = $$optionTargetConfigGroup[i]
          const optionTargetConfigsLen = optionTargetConfigs.length
          if (optionTargetConfigsLen > 0) {
            await this.getOptions(optionTargetConfigs, optionGroup)
          }
        }
      }

      return Object.freeze(optionGroup)
    },
    setOptionGroup () {
      this.getOptionGroup(target).then(optionGroup => {
        this.optionGroup = optionGroup
        target.$$optionData = optionGroup
        this.loadList()
      })
    }
  },
  created () {
    this.setOptionGroup()
  }
})
